part of masamune;

/// Create an extension method for [AppRef] to handle the Query for the model.
///
/// モデル用のQueryを処理するための[AppRef]の拡張メソッドを作成します。
extension MasamunePrefsAppRefExtensions on AppRef {
  /// Retrieve the state-preserved [TModel] by passing the [ModelQueryBase] code generated by the builder.
  ///
  /// ビルダーによりコード生成された[ModelQueryBase]を渡すことにより状態を保持された[TModel]を取得します。
  ///
  /// ```dart
  /// final document = appRef.model(UserModel.document()); // Get the user document.
  /// final collection = appRef.model(UserModel.collection()); // Get the user collection.
  /// ```
  TPrefs prefs<TPrefs extends PrefsBase>(
    TPrefs Function() query,
  ) {
    return watch((ref) => query.call());
  }
}

/// Create extension methods for [PageRef] and [WidgetRef] to handle Query for models.
///
/// モデル用のQueryを処理するための[PageRef]や[WidgetRef]の拡張メソッドを作成します。
extension MasamunePrefsExtensions on RefHasApp {
  /// Retrieve the state-preserved [TModel] by passing the [ModelQueryBase] code generated by the builder.
  ///
  /// Any changes to the model are monitored and the widgets used are updated when changes are made.
  ///
  /// ビルダーによりコード生成された[ModelQueryBase]を渡すことにより状態を保持された[TModel]を取得します。
  ///
  /// モデルの変更はすべて監視され、変更が行われた際、利用したウィジェットは更新されます。
  ///
  /// ```dart
  /// final document = ref.model(UserModel.document()); // Get the user document.
  /// final collection = ref.model(UserModel.collection()); // Get the user collection.
  /// ```
  TPrefs prefs<TPrefs extends PrefsBase>(
    TPrefs Function() query,
  ) {
    return app.watch((ref) => query.call());
  }
}
